package lab.gui;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import javafx.collections.ListChangeListener;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.SelectionMode;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.TextField;
import javafx.scene.control.ToggleButton;
import javafx.scene.control.ToggleGroup;
import javafx.scene.control.cell.PropertyValueFactory;
import lab.Setup;
import lab.data.Score;
import lab.game.Difficult;

/**
 * 
 */
public class MainScreenController {

	private static Logger log = LogManager.getLogger(MainScreenController.class);

	@FXML
	private Button btnGenerateScore;

	@FXML
	private Button btnLoadAll;

	@FXML
	private Button btnLoadFirstTen;

	@FXML
	private Button btnDelete;


	@FXML
	private TableView<Score> scores;

	@FXML
	private TableColumn<Score, String> nickColumn;

	@FXML
	private TableColumn<Score, Integer> pointsColumn;

	@FXML
	private TableColumn<Score, Integer> difficultColumn;

	@FXML
	private ToggleGroup difficult;

	@FXML
	private ToggleButton easy;

	@FXML
	private ToggleButton hard;

	@FXML
	private ToggleButton medium;

	@FXML
	private TextField name;

	private App app;

	@FXML
	void play(ActionEvent event) throws IOException {
		Difficult dif = (Difficult) difficult.getSelectedToggle().getProperties().get(Difficult.class);
		app.switchToGame(name.getText(), dif.getNumberOfMonsters());
	}

	public void setApp(App app) {
		this.app = app;
	}

	@FXML
	void btnGenerateScoreAction(ActionEvent event) {
		Score score = Score.generate();
		this.scores.getItems().add(score);
		Setup.getInstance().getScoreStorageInterface().save(score);
	}

	@FXML
	void btnLoadAllAction(ActionEvent event) {
		updateScoreTable(Setup.getInstance().getScoreStorageInterface().getAll());
	}

	@FXML
	void btnLoadFirstTenAction(ActionEvent event) {
		updateScoreTable(Setup.getInstance().getScoreStorageInterface().getFirstTen());
	}
	
	@FXML
	void btnDeleteAction(ActionEvent event) {
		List<Score> selectedScores = new ArrayList<>(scores.getSelectionModel().getSelectedItems());
		Setup.getInstance().getScoreStorageInterface().delete(selectedScores);
		updateScoreTable(Setup.getInstance().getScoreStorageInterface().getAll());
	}

	private void updateScoreTable(List<Score> scores) {
		this.scores.getItems().clear();
		this.scores.getItems().addAll(scores);
	}

	@FXML
	void initialize() {
		assert difficult != null : "fx:id=\"difficult\" was not injected: check your FXML file 'mainScreen.fxml'.";
		assert name != null : "fx:id=\"name\" was not injected: check your FXML file 'mainScreen.fxml'.";
		easy.getProperties().put(Difficult.class, Difficult.EASY);
		medium.getProperties().put(Difficult.class, Difficult.MEDIUM);
		hard.getProperties().put(Difficult.class, Difficult.HARD);

		nickColumn.setCellValueFactory(new PropertyValueFactory<>("name"));
		pointsColumn.setCellValueFactory(new PropertyValueFactory<>("points"));
		difficultColumn.setCellValueFactory(new PropertyValueFactory<>("level"));

		btnDelete.setDisable(true);
		scores.getSelectionModel().setSelectionMode(SelectionMode.MULTIPLE);
		scores.getSelectionModel().getSelectedItems()
				.addListener((ListChangeListener.Change<? extends Score> change) -> 
					btnDelete.setDisable(change.getList().isEmpty()));

		initStorage();
		log.info("Screeen initialized.");
	}

	private void initStorage() {
		Setup.getInstance().getScoreStorageInterface().init();
		scores.getItems().addAll(Setup.getInstance().getScoreStorageInterface().getAll());
	}

}
